/* args.h
 *
 * WMAppl - Window Maker Application Launcher
 * Version:  0.5
 * Copyright (C) 1999-2000 Casey Harkins
 *
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __ARGS_H__

#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* defaults macros */
#define WMAPPLRC ".wmapplrc"
#define ICONPATH "/usr/share/icons/wmappl:/usr/share/pixmaps/mini"


/* MACRO defining the path separator */
#define PATHSEP     ':'
/* MACRO defining the directory separator (separating each individual directory) */
#define DIRSEP      '/'

/* icon path structure */
typedef struct _IconPath_ {
    int count;
    size_t dirallocsize;
    char **path;
} IconPath;


/* parse_args
 *	PARAMETERS:
 *      int argc          : number of command line parameters
 *      char **argv       : array of command line parameters
 *	RETURN:
 *		none
 */
void
parse_args(int argc, char **argv);

/* show_version
 *	PARAMETERS:
 *		none
 *	RETURN:
 *		none
 */
void
show_version();

/* show_usage
 *	PARAMETERS:
 *		none
 *	RETURN:
 *		none
 */
void
show_usage();

/* get_rcfile
 *	PARAMETERS:
 *      none
 *	RETURN:
 *		char *	: resource filename, should not return NULL
 */
char *
get_rcfile();

/* get_iconpath
 *	PARAMETERS:
 *      none
 *	RETURN:
 *		IconPath *	: the icon path
 */
IconPath *
get_iconpath();

/* create_iconpath
 *  PARAMETERS:
 *      char * : colon separated path
 *  RETURN:
 *      IconPath *  : the icon path as a structure 
*/
IconPath *
create_iconpath();

/* get_debugmode
 *	PARAMETERS:
 *      none
 *	RETURN:
 *		int		: one if debug option was set, else zero	
 */
int
get_debugmode();

#endif /* not __ARGS_H__ */
