/*
 * WMAppl - Window Maker Application Launcher
 * Version:  0.6
 * Copyright (C) 1999-2000 Casey Harkins
 *
 * Contributors:
 *		Casey Harkins (charkins@pobox.com)
 *		Richard Eisenman 
 *		Jacob Beardsley	
 *		Steve Akers
 *		Samoylov Olleg
 *		Jarek (see dockapp.c)
 *		Anyone else that I may have missed!
 *
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include "interface.xpm"
#include "leftarr.xpm"
#include "rightarr.xpm"
#include "dockapp.h"
#include "args.h"
#include "xpmbutton.h"
#include "rcparser.h"
#include "fileexists.h"

XpmButton *icons, *sleft, *sright, *pressed;
int clearwindow = 0;

int button_positions[] = {5, 5,
                          5, 23,
						  23, 5, 
						  23, 23,
						  41, 5,
						  41, 23};

int scroll_positions[] = {5, 46,
                          42, 46};


void
set_button_positions()
{
	XpmButton *b;
	int i;

	for(i=0, b=icons; i<6 && b!=NULL; i++, b=b->next) {
		set_xpmbutton_position(b, button_positions[i*2], button_positions[(i*2)+1]);
	}
}

/* paint callback */
void
wmappl_paint(Display *dsp, Drawable drw, GC gc)
{
	XpmButton *p;
	int i;

	/* clear the window if needed */
	if(clearwindow) {
		clear_window();
		/* reset the clearwindow variable */
		clearwindow=0;
	}

	/* draw the six visible buttons */
	for(i=0, p=icons; i<6 && p!=NULL; i++, p=p->next)
		draw_xpmbutton(p, dsp, drw, gc);

	/* draw the scroll buttons */
	draw_xpmbutton(sleft, dsp, drw, gc);
	draw_xpmbutton(sright, dsp, drw, gc);

}

/* mouse callback */
void
wmappl_mouse(int x, int y, int z, int s)
{
	int i;
	XpmButton *p=NULL, *released=NULL;

	if(s==MOUSE_PRESSED) {
		/* clear pressed marker if it is pointing somewhere */
		if(pressed!=NULL) {
			set_xpmbutton_pressed(pressed, 0);
			pressed=NULL;
		}

		/* check visible icons */
		for(i=0, p=icons; i<6 && p!=NULL; i++, p=p->next) {
			if(is_xpmbutton_at(p, x, y)) pressed = p;
		}

		/* check scroll buttons */
		if(is_xpmbutton_at(sleft, x, y)) pressed = sleft;
		if(is_xpmbutton_at(sright, x, y)) pressed = sright;

		/* if a button was pressed, mark it and request repaint */
		if(pressed!=NULL) {
			set_xpmbutton_pressed(pressed, 1);
			request_paint();
		}
	}

	if(s==MOUSE_RELEASED) {
		/* ignore releases if a button hasn't been pressed */
		if(!pressed) return;

		/* check visible icons */
		for(i=0, p=icons; i<6 && p!=NULL; i++, p=p->next) {
			if(is_xpmbutton_at(p, x, y)) released=p;
		}

		/* check scroll buttons */
		if(is_xpmbutton_at(sleft, x, y)) released = sleft;
		if(is_xpmbutton_at(sright, x, y)) released = sright;

		/* if the pressed button was released perform action */
		if(pressed == released) {
		
			/* scroll left */
			if(pressed == sleft) {
				if(icons->last!=NULL) {
					if(icons->last->last!=NULL) {
						icons=icons->last->last;
					  	clearwindow=1;
						set_button_positions();
					}
				}
			}
			
			/* scroll right */
			else if(pressed == sright) {
				/* the following nested if is not indented
				 * properly but at least it < 80 chars wide */
				if(icons->next!=NULL) {
				 if(icons->next->next!=NULL) {
				  if(icons->next->next->next!=NULL) {
				   if(icons->next->next->next->next!=NULL) {
				    if(icons->next->next->next->next->next!=NULL) {
				     if(icons->next->next->next->next->next->next!=NULL) {
				      icons=icons->next->next;
					  clearwindow=1;
					  set_button_positions();
					 }
				    }
				   }
				  }
				 }
				}
			}
			
			/* execute command associated with button */
			else {
				if(pressed->command!=NULL) {
					if(get_debugmode()) {
						fprintf(stderr,"Execute command: %s\n",pressed->command);
					}
					else {
						system(pressed->command);
					}
				}
			}

		}
		
		/* unmark button pressed and request repaint */
		set_xpmbutton_pressed(pressed, 0);
		request_paint();

		/* clear pressed and released pointers */
		pressed = NULL;
		released = NULL;
	}
}

char *
get_abspath_to_icon(char *filename, IconPath *iconpath)
{
	char *abspath;
	int i;

	if(!filename || !iconpath) return (char *)NULL;


	/* check if filename is absolute path */
	if(fileexists(filename)) {
			
		/* allocate return value */
		abspath = (char *)malloc(sizeof(char)*(strlen(filename)+1));
		if(!abspath) {
			fprintf(stderr, "wmappl.c: Couldn't allocate string.\n");
			exit(1);
		}
		
		/* copy the filename into the return value */
		strcpy(abspath, filename);
		
		return abspath;
	}

	abspath = (char *)malloc(iconpath->dirallocsize+(sizeof(char)*strlen(filename)));

	/* try each directory in the IconPath structure */
	for(i=0; i<iconpath->count; i++) {
		
		/* copy the directory into the absolute path */
		strcpy(abspath, iconpath->path[i]);
		strcat(abspath, filename);

		/* check to see if filename exists at the absolute path */
		if(fileexists(abspath)) {

			/* reallocate return value to it's proper size */
			abspath = (char *)realloc(abspath, sizeof(char)*(strlen(abspath)+1));
			if(!abspath) {
				fprintf(stderr, "wmappl.c: Couldn't reallocate string.\n");
				exit(1);
			}

			return abspath;
		}

	}

	return (char *)NULL;
}


void
load_icons()
{
	RCFile *rcfile;
	IconPath *iconpath;
	int i, j;
	XpmButton *first = NULL;
	char *iconname=NULL, *command=NULL;

	/* load the scroll icons */
	sleft = create_include_xpmbutton(leftarr_xpm, get_display());
	sright = create_include_xpmbutton(rightarr_xpm, get_display());

	/* error out if scroll arrows couldn't be loaded */
	if(!sleft || !sright) {
		fprintf(stderr, "wmappl.c: Couldn't create scroll buttons.\n");
		exit(1);
	}
	
	/* set the initial positions of the scroll arrows */
	set_xpmbutton_position(sleft, scroll_positions[0], scroll_positions[1]);
	set_xpmbutton_position(sright, scroll_positions[2], scroll_positions[3]);

	/* load the rcfile into an RCFile structure */
	rcfile = parse_rcfile(get_rcfile());

	/* load the icon path from args */
	iconpath = get_iconpath();

	/* traverse the structure, loading each icon */
	for(i=0; i<rcfile->section_count; i++) {
			
		/* reset command pointer */
		command = NULL;
		iconname = NULL;

		for(j=0; j<rcfile->section[i]->key_count; j++) {
			if(strcmp(rcfile->section[i]->key[j]->key, "icon")==0) {
				iconname = get_abspath_to_icon(rcfile->section[i]->key[j]->value, iconpath);
				if(!iconname) {
					fprintf(stderr, "wmappl.c: Couldn't find %s in the iconpath.\n", rcfile->section[i]->key[j]->value);
					exit(1);
				}
			}
			if(strcmp(rcfile->section[i]->key[j]->key, "command")==0) {
				command = rcfile->section[i]->key[j]->value;
			}
		}

		if(iconname && command) {
			if(!first) {
				/* create the xpmbutton */
				first = create_xpmbutton(iconname, get_display());
				if(!first) {
					fprintf(stderr, "wmappl.c: Couldn't load icon %s.\n", iconname);
					exit(1);
				}

				/* set the command */
				set_xpmbutton_command(first, command);

				/* warn if icon isn't 16x16 (18x18 counting added border) */
				if(first->width!=18 || first->height!=18) {
					fprintf(stderr, "wmappl.c: Warning, %s is not 16x16 and may cause undesired results.\n", iconname);
				}
				
				/* set the icons pointer to the first icon */
				icons = first;

			}
			else {
				/* create the xpmbutton */
				icons->next = create_xpmbutton(iconname, get_display());
				if(!icons) {
					fprintf(stderr, "wmappl.c: Couldn't load icon %s.\n", iconname);
					exit(1);
				}

				/* set the command */
				set_xpmbutton_command(icons->next, command);

				/* warn if icon isn't 16x16 (18x18 counting added border) */
				if(icons->next->width!=18 || icons->next->height!=18) {
					fprintf(stderr, "wmappl.c: Warning, %s is not 16x16 and may cause undesired results.\n", iconname);
				}
				
				/* set linked list pointers */
				icons->next->last = icons;
				icons = icons->next;
			}

			/* free allocated absolute icon filename */
			free(iconname);

		}
		else {
			if(iconname) fprintf(stderr, "wmappl.c: You must specify a command for the icon %s.\n", iconname);
			exit(1);
		}

	}

	/* set the icons pointer to the first icon */
	icons = first;

	/* set the button positions */
	set_button_positions();

	/* free the RCFile structure */
	free_rcfile(rcfile);

}

	
int
main(int argc, char **argv)
{
	/* parse the command line arguments */
	parse_args(argc, argv);

	/* create the dockapp */
	if(!create_dockapp("WMAppl","64x64+0+0",interface_xpm,1,argc,argv)) {
		fprintf(stderr, "Couldn't create dockapp.\n");
		exit(1);
	}

	/* load the icons */
	load_icons();

	/* set the paint and mouse callbacks */
	set_paint_func(wmappl_paint);
	set_mouse_func(wmappl_mouse);

	/* enter the dockapp event loop */
	run_dockapp();
	
	return 0;
}
